/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <tice.h>

/* Standard headers */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Shared libraries */
#include <lib/ce/graphx.h>
#include <lib/ce/fileioc.h>
#include <lib/ce/keypadc.h>
#include "gfx/player_gfx.h"
#include "gfx/tile_gfx.h"

/* Tilemap defines */
#define TILE_WIDTH	11
#define TILE_HEIGHT	11
#define TILEMAP_WIDTH	30
#define TILEMAP_HEIGHT	18
#define TILEMAP_X_OFFSET -5
#define TILEMAP_Y_OFFSET -5

/* Palette defines */
#define SPIKE_COLOR	107
#define BLOCK_COLOR	227
#define WHITE		255

/* Other defines */
#define SPAWN_X	50
#define SPAWN_Y	50
#define GRAVITY_VELOCITY	0.21
#define JUMP_VELOCITY	-4.03

extern uint8_t TITOL_map[];

const gfx_image_t *tile_ptrs[] = {
   block, block, up_spike, down_spike, left_spike, right_spike, start_pipe, end_pipe, door, door, button
};

const char *stage_names[] = {
	"Arrow keys required",
	"Not always straight forward",
	"Think before doing",
	"Alternate control scheme",
	"Freefloating",
};

uint8_t temp_x, temp_y;

uint8_t stage = 1;
int deaths = 0;

/* Put function prototypes here */
uint8_t collision( uint8_t tile_x, uint8_t tile_y );
void draw_tile( uint8_t tile_x, uint8_t tile_y );
void draw_tilemap(void);
bool hit_wall( int hit_x, int hit_y );

/* Put all your code here */
void main(void) {
	int player_x;
	double player_y;
	double player_y_velocity;
	uint8_t block_color[3], spike_color[3];
	uint8_t key7;
	gfx_image_t *player_back;
		
	/* Seed the random number generator */
	srand(rtc_Time());
	
	/* Allocate space for the temporary sprite to save the background */
	malloc(0);
	player_back = gfx_AllocSprite( player->width, player->height, malloc );
	
	/* Initialize graphics */
	gfx_Begin(gfx_8bpp);
	
	/* Set up the palette */
	gfx_SetPalette(tile_gfx_pal, sizeof(tile_gfx_pal), 0);
	gfx_SetDrawBuffer();
	
	/* Program code*/
	
	player_x = SPAWN_X;
	player_y = SPAWN_Y;
	
	player_y_velocity = 0.0;
	
	//gfx_palette[BLOCK_COLOR] = randInt( 0x100, 0xF000 );
	//gfx_palette[SPIKE_COLOR] = randInt( 0x100, 0xF000 );
	gfx_palette[BLOCK_COLOR] = gfx_RGBTo1555( randInt( 0, 200 ), randInt( 0, 200 ), randInt( 0, 200 ) );
	gfx_palette[SPIKE_COLOR] = gfx_RGBTo1555( randInt( 0, 200 ), randInt( 0, 200 ), randInt( 0, 200 ) );
	
	draw_tilemap();
	
	gfx_SetColor( gfx_palette[58] );
	gfx_FillRectangle_NoClip( 0, 188, 320, 52 );
	
	gfx_SetTextScale( 1, 1 );
	gfx_PrintStringXY( "Level 1   Stage ", 2, 191 );
	gfx_PrintInt( stage, 1 + (stage >= 10) );
	gfx_PrintStringXY( stage_names[stage - 1], 2, 200 );
	
	gfx_GetSprite( player_back, player_x, player_y );
	
	while( kb_ScanGroup(kb_group_6) != kb_Clear ) {
		
		gfx_Sprite_NoClip( player_back, player_x, player_y );
		
		if( (key7 = kb_ScanGroup(kb_group_7))) {
			if ( key7 & kb_Left ) {
				if ( !hit_wall( player_x - 1, player_y ) ) {
					player_x--;
				}
			}
			if ( key7 & kb_Right ) {
				if ( !hit_wall( player_x + 1, player_y ) ) {
					player_x++;
				}
			}
		}
		if ( hit_wall( player_x, player_y + TILE_HEIGHT ) ) {
			if ( key7 & kb_Up ) {
				player_y_velocity = JUMP_VELOCITY;
			} else {
				player_y -= TILEMAP_Y_OFFSET;
				player_y = floor( player_y / TILE_WIDTH ) * TILE_WIDTH;
				player_y += TILEMAP_Y_OFFSET;
				player_y_velocity = 0.0;
			}
		} else {
			player_y_velocity += GRAVITY_VELOCITY;
		}
		
		player_y += player_y_velocity;
		
		gfx_GetSprite_NoClip( player_back, player_x, player_y );
		gfx_TransparentSprite_NoClip( player, player_x, player_y );
		gfx_Blit( gfx_buffer );
	}
	
	/* Clean up and exit */
	gfx_End(); prgm_CleanUp();
}

uint8_t collision( uint8_t tile_x, uint8_t tile_y ) {
	uint8_t tile = TITOL_map[TILEMAP_WIDTH * tile_y + tile_x];
	if ( tile == 1 || tile == 8 || tile == 9 ) { return 1; }
	if ( tile >= 2 & tile <= 5 ) { return 2; }
	return 0;
}

void draw_tile( uint8_t tile_x, uint8_t tile_y ) {
	unsigned xpos = TILE_WIDTH * tile_x + TILEMAP_X_OFFSET;
	unsigned ypos = TILE_HEIGHT * tile_y + TILEMAP_Y_OFFSET;
	uint8_t tile = TITOL_map[TILEMAP_WIDTH * tile_y + tile_x];
	
	switch( tile ) {
		case 0:
		case 11:
			break;
		case 1:
			gfx_Sprite( block, xpos, ypos );
			break;
		case 8:
		case 9:
			gfx_TransparentSprite_NoClip( door, xpos, ypos );
			break;
		default:
			gfx_TransparentSprite_NoClip( tile_ptrs[tile], xpos, ypos );
			break;
	}
}

void draw_tilemap(void) {
	uint8_t tilemap_x, tilemap_y;
	
	gfx_FillScreen( WHITE );
	for( tilemap_x = 0; tilemap_x < TILEMAP_WIDTH; tilemap_x++ ) {
		for( tilemap_y = 0; tilemap_y < TILEMAP_HEIGHT; tilemap_y++ ) {
			draw_tile( tilemap_x, tilemap_y );
		}
	}
}

bool hit_wall( int hit_x, int hit_y ) {
	bool hit_wall = false;
	for( temp_x = 0; temp_x < TILE_WIDTH - 1; temp_x += TILE_WIDTH - 1 ) {
		for( temp_y = 0; temp_y < TILE_HEIGHT - 1; temp_y += TILE_HEIGHT - 1 ) {
			if (collision( (hit_x + temp_x - TILEMAP_X_OFFSET) / TILE_WIDTH, (hit_y + temp_y - TILEMAP_Y_OFFSET) / TILE_HEIGHT ) == 1) {
				hit_wall = true;
			}
		}
	}
	return hit_wall;
}